from .base_classes import *
from .support_functions import *


class Patient(Resource):
    """Demographics and other administrative information about a person or
    animal receiving care or other health-related services.If the
    element is present, it must have either a @value, an @id, or
    extensionsIndicates if the individual is deceased or
    not.Indicates whether the patient is part of a multiple or
    indicates the actual birth order."""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, name=None, telecom=None, gender=None, birthDate=None, deceasedBoolean=None, deceasedDateTime=None, address=None, maritalStatus=None, multipleBirthBoolean=None, multipleBirthInteger=None, photo=None, contact=None, animal=None, communication=None, careProvider=None, managingOrganization=None, link=None, active=None):
        self.original_tagname_ = None
        super(Patient, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        if name is None:
            self.name = []
        else:
            self.name = name
        if telecom is None:
            self.telecom = []
        else:
            self.telecom = telecom
        self.gender = gender
        self.birthDate = birthDate
        self.deceasedBoolean = deceasedBoolean
        self.deceasedDateTime = deceasedDateTime
        if address is None:
            self.address = []
        else:
            self.address = address
        self.maritalStatus = maritalStatus
        self.multipleBirthBoolean = multipleBirthBoolean
        self.multipleBirthInteger = multipleBirthInteger
        if photo is None:
            self.photo = []
        else:
            self.photo = photo
        if contact is None:
            self.contact = []
        else:
            self.contact = contact
        self.animal = animal
        if communication is None:
            self.communication = []
        else:
            self.communication = communication
        if careProvider is None:
            self.careProvider = []
        else:
            self.careProvider = careProvider
        self.managingOrganization = managingOrganization
        if link is None:
            self.link = []
        else:
            self.link = link
        self.active = active
    def factory(*args_, **kwargs_):
        if Patient.subclass:
            return Patient.subclass(*args_, **kwargs_)
        else:
            return Patient(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def add_name(self, value): self.name.append(value)
    def insert_name(self, index, value): self.name[index] = value
    def get_telecom(self): return self.telecom
    def set_telecom(self, telecom): self.telecom = telecom
    def add_telecom(self, value): self.telecom.append(value)
    def insert_telecom(self, index, value): self.telecom[index] = value
    def get_gender(self): return self.gender
    def set_gender(self, gender): self.gender = gender
    def get_birthDate(self): return self.birthDate
    def set_birthDate(self, birthDate): self.birthDate = birthDate
    def get_deceasedBoolean(self): return self.deceasedBoolean
    def set_deceasedBoolean(self, deceasedBoolean): self.deceasedBoolean = deceasedBoolean
    def get_deceasedDateTime(self): return self.deceasedDateTime
    def set_deceasedDateTime(self, deceasedDateTime): self.deceasedDateTime = deceasedDateTime
    def get_address(self): return self.address
    def set_address(self, address): self.address = address
    def add_address(self, value): self.address.append(value)
    def insert_address(self, index, value): self.address[index] = value
    def get_maritalStatus(self): return self.maritalStatus
    def set_maritalStatus(self, maritalStatus): self.maritalStatus = maritalStatus
    def get_multipleBirthBoolean(self): return self.multipleBirthBoolean
    def set_multipleBirthBoolean(self, multipleBirthBoolean): self.multipleBirthBoolean = multipleBirthBoolean
    def get_multipleBirthInteger(self): return self.multipleBirthInteger
    def set_multipleBirthInteger(self, multipleBirthInteger): self.multipleBirthInteger = multipleBirthInteger
    def get_photo(self): return self.photo
    def set_photo(self, photo): self.photo = photo
    def add_photo(self, value): self.photo.append(value)
    def insert_photo(self, index, value): self.photo[index] = value
    def get_contact(self): return self.contact
    def set_contact(self, contact): self.contact = contact
    def add_contact(self, value): self.contact.append(value)
    def insert_contact(self, index, value): self.contact[index] = value
    def get_animal(self): return self.animal
    def set_animal(self, animal): self.animal = animal
    def get_communication(self): return self.communication
    def set_communication(self, communication): self.communication = communication
    def add_communication(self, value): self.communication.append(value)
    def insert_communication(self, index, value): self.communication[index] = value
    def get_careProvider(self): return self.careProvider
    def set_careProvider(self, careProvider): self.careProvider = careProvider
    def add_careProvider(self, value): self.careProvider.append(value)
    def insert_careProvider(self, index, value): self.careProvider[index] = value
    def get_managingOrganization(self): return self.managingOrganization
    def set_managingOrganization(self, managingOrganization): self.managingOrganization = managingOrganization
    def get_link(self): return self.link
    def set_link(self, link): self.link = link
    def add_link(self, value): self.link.append(value)
    def insert_link(self, index, value): self.link[index] = value
    def get_active(self): return self.active
    def set_active(self, active): self.active = active
    def hasContent_(self):
        if (
            self.identifier or
            self.name or
            self.telecom or
            self.gender is not None or
            self.birthDate is not None or
            self.deceasedBoolean is not None or
            self.deceasedDateTime is not None or
            self.address or
            self.maritalStatus is not None or
            self.multipleBirthBoolean is not None or
            self.multipleBirthInteger is not None or
            self.photo or
            self.contact or
            self.animal is not None or
            self.communication or
            self.careProvider or
            self.managingOrganization is not None or
            self.link or
            self.active is not None or
            super(Patient, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Patient', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Patient')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Patient', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Patient'):
        super(Patient, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Patient')
    def exportChildren(self, outfile, level, namespace_='', name_='Patient', fromsubclass_=False, pretty_print=True):
        super(Patient, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        for name_ in self.name:
            name_.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        for telecom_ in self.telecom:
            telecom_.export(outfile, level, namespace_, name_='telecom', pretty_print=pretty_print)
        if self.gender is not None:
            self.gender.export(outfile, level, namespace_, name_='gender', pretty_print=pretty_print)
        if self.birthDate is not None:
            self.birthDate.export(outfile, level, namespace_, name_='birthDate', pretty_print=pretty_print)
        if self.deceasedBoolean is not None:
            self.deceasedBoolean.export(outfile, level, namespace_, name_='deceasedBoolean', pretty_print=pretty_print)
        if self.deceasedDateTime is not None:
            self.deceasedDateTime.export(outfile, level, namespace_, name_='deceasedDateTime', pretty_print=pretty_print)
        for address_ in self.address:
            address_.export(outfile, level, namespace_, name_='address', pretty_print=pretty_print)
        if self.maritalStatus is not None:
            self.maritalStatus.export(outfile, level, namespace_, name_='maritalStatus', pretty_print=pretty_print)
        if self.multipleBirthBoolean is not None:
            self.multipleBirthBoolean.export(outfile, level, namespace_, name_='multipleBirthBoolean', pretty_print=pretty_print)
        if self.multipleBirthInteger is not None:
            self.multipleBirthInteger.export(outfile, level, namespace_, name_='multipleBirthInteger', pretty_print=pretty_print)
        for photo_ in self.photo:
            photo_.export(outfile, level, namespace_, name_='photo', pretty_print=pretty_print)
        for contact_ in self.contact:
            contact_.export(outfile, level, namespace_, name_='contact', pretty_print=pretty_print)
        if self.animal is not None:
            self.animal.export(outfile, level, namespace_, name_='animal', pretty_print=pretty_print)
        for communication_ in self.communication:
            communication_.export(outfile, level, namespace_, name_='communication', pretty_print=pretty_print)
        for careProvider_ in self.careProvider:
            careProvider_.export(outfile, level, namespace_, name_='careProvider', pretty_print=pretty_print)
        if self.managingOrganization is not None:
            self.managingOrganization.export(outfile, level, namespace_, name_='managingOrganization', pretty_print=pretty_print)
        for link_ in self.link:
            link_.export(outfile, level, namespace_, name_='link', pretty_print=pretty_print)
        if self.active is not None:
            self.active.export(outfile, level, namespace_, name_='active', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Patient', mapping_=None):
        element = super(Patient, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        for name_ in self.name:
            name_.to_etree(element, name_='name', mapping_=mapping_)
        for telecom_ in self.telecom:
            telecom_.to_etree(element, name_='telecom', mapping_=mapping_)
        if self.gender is not None:
            gender_ = self.gender
            gender_.to_etree(element, name_='gender', mapping_=mapping_)
        if self.birthDate is not None:
            birthDate_ = self.birthDate
            birthDate_.to_etree(element, name_='birthDate', mapping_=mapping_)
        if self.deceasedBoolean is not None:
            deceasedBoolean_ = self.deceasedBoolean
            deceasedBoolean_.to_etree(element, name_='deceasedBoolean', mapping_=mapping_)
        if self.deceasedDateTime is not None:
            deceasedDateTime_ = self.deceasedDateTime
            deceasedDateTime_.to_etree(element, name_='deceasedDateTime', mapping_=mapping_)
        for address_ in self.address:
            address_.to_etree(element, name_='address', mapping_=mapping_)
        if self.maritalStatus is not None:
            maritalStatus_ = self.maritalStatus
            maritalStatus_.to_etree(element, name_='maritalStatus', mapping_=mapping_)
        if self.multipleBirthBoolean is not None:
            multipleBirthBoolean_ = self.multipleBirthBoolean
            multipleBirthBoolean_.to_etree(element, name_='multipleBirthBoolean', mapping_=mapping_)
        if self.multipleBirthInteger is not None:
            multipleBirthInteger_ = self.multipleBirthInteger
            multipleBirthInteger_.to_etree(element, name_='multipleBirthInteger', mapping_=mapping_)
        for photo_ in self.photo:
            photo_.to_etree(element, name_='photo', mapping_=mapping_)
        for contact_ in self.contact:
            contact_.to_etree(element, name_='contact', mapping_=mapping_)
        if self.animal is not None:
            animal_ = self.animal
            animal_.to_etree(element, name_='animal', mapping_=mapping_)
        for communication_ in self.communication:
            communication_.to_etree(element, name_='communication', mapping_=mapping_)
        for careProvider_ in self.careProvider:
            careProvider_.to_etree(element, name_='careProvider', mapping_=mapping_)
        if self.managingOrganization is not None:
            managingOrganization_ = self.managingOrganization
            managingOrganization_.to_etree(element, name_='managingOrganization', mapping_=mapping_)
        for link_ in self.link:
            link_.to_etree(element, name_='link', mapping_=mapping_)
        if self.active is not None:
            active_ = self.active
            active_.to_etree(element, name_='active', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Patient, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'name':
            obj_ = HumanName.factory()
            obj_.build(child_)
            self.name.append(obj_)
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'telecom':
            obj_ = Contact.factory()
            obj_.build(child_)
            self.telecom.append(obj_)
            obj_.original_tagname_ = 'telecom'
        elif nodeName_ == 'gender':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.gender = obj_
            obj_.original_tagname_ = 'gender'
        elif nodeName_ == 'birthDate':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.birthDate = obj_
            obj_.original_tagname_ = 'birthDate'
        elif nodeName_ == 'deceasedBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.deceasedBoolean = obj_
            obj_.original_tagname_ = 'deceasedBoolean'
        elif nodeName_ == 'deceasedDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.deceasedDateTime = obj_
            obj_.original_tagname_ = 'deceasedDateTime'
        elif nodeName_ == 'address':
            obj_ = Address.factory()
            obj_.build(child_)
            self.address.append(obj_)
            obj_.original_tagname_ = 'address'
        elif nodeName_ == 'maritalStatus':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.maritalStatus = obj_
            obj_.original_tagname_ = 'maritalStatus'
        elif nodeName_ == 'multipleBirthBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.multipleBirthBoolean = obj_
            obj_.original_tagname_ = 'multipleBirthBoolean'
        elif nodeName_ == 'multipleBirthInteger':
            obj_ = integer.factory()
            obj_.build(child_)
            self.multipleBirthInteger = obj_
            obj_.original_tagname_ = 'multipleBirthInteger'
        elif nodeName_ == 'photo':
            obj_ = Attachment.factory()
            obj_.build(child_)
            self.photo.append(obj_)
            obj_.original_tagname_ = 'photo'
        elif nodeName_ == 'contact':
            obj_ = Patient_Contact.factory()
            obj_.build(child_)
            self.contact.append(obj_)
            obj_.original_tagname_ = 'contact'
        elif nodeName_ == 'animal':
            obj_ = Patient_Animal.factory()
            obj_.build(child_)
            self.animal = obj_
            obj_.original_tagname_ = 'animal'
        elif nodeName_ == 'communication':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.communication.append(obj_)
            obj_.original_tagname_ = 'communication'
        elif nodeName_ == 'careProvider':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.careProvider.append(obj_)
            obj_.original_tagname_ = 'careProvider'
        elif nodeName_ == 'managingOrganization':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.managingOrganization = obj_
            obj_.original_tagname_ = 'managingOrganization'
        elif nodeName_ == 'link':
            obj_ = Patient_Link.factory()
            obj_.build(child_)
            self.link.append(obj_)
            obj_.original_tagname_ = 'link'
        elif nodeName_ == 'active':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.active = obj_
            obj_.original_tagname_ = 'active'
        super(Patient, self).buildChildren(child_, node, nodeName_, True)
# end class Patient


class Patient_Contact(BackboneElement):
    """Demographics and other administrative information about a person or
    animal receiving care or other health-related services."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, relationship=None, name=None, telecom=None, address=None, gender=None, organization=None):
        self.original_tagname_ = None
        super(Patient_Contact, self).__init__(id, extension, modifierExtension, )
        if relationship is None:
            self.relationship = []
        else:
            self.relationship = relationship
        self.name = name
        if telecom is None:
            self.telecom = []
        else:
            self.telecom = telecom
        self.address = address
        self.gender = gender
        self.organization = organization
    def factory(*args_, **kwargs_):
        if Patient_Contact.subclass:
            return Patient_Contact.subclass(*args_, **kwargs_)
        else:
            return Patient_Contact(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_relationship(self): return self.relationship
    def set_relationship(self, relationship): self.relationship = relationship
    def add_relationship(self, value): self.relationship.append(value)
    def insert_relationship(self, index, value): self.relationship[index] = value
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_telecom(self): return self.telecom
    def set_telecom(self, telecom): self.telecom = telecom
    def add_telecom(self, value): self.telecom.append(value)
    def insert_telecom(self, index, value): self.telecom[index] = value
    def get_address(self): return self.address
    def set_address(self, address): self.address = address
    def get_gender(self): return self.gender
    def set_gender(self, gender): self.gender = gender
    def get_organization(self): return self.organization
    def set_organization(self, organization): self.organization = organization
    def hasContent_(self):
        if (
            self.relationship or
            self.name is not None or
            self.telecom or
            self.address is not None or
            self.gender is not None or
            self.organization is not None or
            super(Patient_Contact, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Patient.Contact', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Patient.Contact')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Patient.Contact', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Patient.Contact'):
        super(Patient_Contact, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Patient.Contact')
    def exportChildren(self, outfile, level, namespace_='', name_='Patient.Contact', fromsubclass_=False, pretty_print=True):
        super(Patient_Contact, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for relationship_ in self.relationship:
            relationship_.export(outfile, level, namespace_, name_='relationship', pretty_print=pretty_print)
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        for telecom_ in self.telecom:
            telecom_.export(outfile, level, namespace_, name_='telecom', pretty_print=pretty_print)
        if self.address is not None:
            self.address.export(outfile, level, namespace_, name_='address', pretty_print=pretty_print)
        if self.gender is not None:
            self.gender.export(outfile, level, namespace_, name_='gender', pretty_print=pretty_print)
        if self.organization is not None:
            self.organization.export(outfile, level, namespace_, name_='organization', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Patient.Contact', mapping_=None):
        element = super(Patient_Contact, self).to_etree(parent_element, name_, mapping_)
        for relationship_ in self.relationship:
            relationship_.to_etree(element, name_='relationship', mapping_=mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        for telecom_ in self.telecom:
            telecom_.to_etree(element, name_='telecom', mapping_=mapping_)
        if self.address is not None:
            address_ = self.address
            address_.to_etree(element, name_='address', mapping_=mapping_)
        if self.gender is not None:
            gender_ = self.gender
            gender_.to_etree(element, name_='gender', mapping_=mapping_)
        if self.organization is not None:
            organization_ = self.organization
            organization_.to_etree(element, name_='organization', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Patient_Contact, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'relationship':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.relationship.append(obj_)
            obj_.original_tagname_ = 'relationship'
        elif nodeName_ == 'name':
            obj_ = HumanName.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'telecom':
            obj_ = Contact.factory()
            obj_.build(child_)
            self.telecom.append(obj_)
            obj_.original_tagname_ = 'telecom'
        elif nodeName_ == 'address':
            obj_ = Address.factory()
            obj_.build(child_)
            self.address = obj_
            obj_.original_tagname_ = 'address'
        elif nodeName_ == 'gender':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.gender = obj_
            obj_.original_tagname_ = 'gender'
        elif nodeName_ == 'organization':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.organization = obj_
            obj_.original_tagname_ = 'organization'
        super(Patient_Contact, self).buildChildren(child_, node, nodeName_, True)
# end class Patient_Contact


class Patient_Animal(BackboneElement):
    """Demographics and other administrative information about a person or
    animal receiving care or other health-related services."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, species=None, breed=None, genderStatus=None):
        self.original_tagname_ = None
        super(Patient_Animal, self).__init__(id, extension, modifierExtension, )
        self.species = species
        self.breed = breed
        self.genderStatus = genderStatus
    def factory(*args_, **kwargs_):
        if Patient_Animal.subclass:
            return Patient_Animal.subclass(*args_, **kwargs_)
        else:
            return Patient_Animal(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_species(self): return self.species
    def set_species(self, species): self.species = species
    def get_breed(self): return self.breed
    def set_breed(self, breed): self.breed = breed
    def get_genderStatus(self): return self.genderStatus
    def set_genderStatus(self, genderStatus): self.genderStatus = genderStatus
    def hasContent_(self):
        if (
            self.species is not None or
            self.breed is not None or
            self.genderStatus is not None or
            super(Patient_Animal, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Patient.Animal', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Patient.Animal')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Patient.Animal', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Patient.Animal'):
        super(Patient_Animal, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Patient.Animal')
    def exportChildren(self, outfile, level, namespace_='', name_='Patient.Animal', fromsubclass_=False, pretty_print=True):
        super(Patient_Animal, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.species is not None:
            self.species.export(outfile, level, namespace_, name_='species', pretty_print=pretty_print)
        if self.breed is not None:
            self.breed.export(outfile, level, namespace_, name_='breed', pretty_print=pretty_print)
        if self.genderStatus is not None:
            self.genderStatus.export(outfile, level, namespace_, name_='genderStatus', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Patient.Animal', mapping_=None):
        element = super(Patient_Animal, self).to_etree(parent_element, name_, mapping_)
        if self.species is not None:
            species_ = self.species
            species_.to_etree(element, name_='species', mapping_=mapping_)
        if self.breed is not None:
            breed_ = self.breed
            breed_.to_etree(element, name_='breed', mapping_=mapping_)
        if self.genderStatus is not None:
            genderStatus_ = self.genderStatus
            genderStatus_.to_etree(element, name_='genderStatus', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Patient_Animal, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'species':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.species = obj_
            obj_.original_tagname_ = 'species'
        elif nodeName_ == 'breed':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.breed = obj_
            obj_.original_tagname_ = 'breed'
        elif nodeName_ == 'genderStatus':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.genderStatus = obj_
            obj_.original_tagname_ = 'genderStatus'
        super(Patient_Animal, self).buildChildren(child_, node, nodeName_, True)
# end class Patient_Animal


class Patient_Link(BackboneElement):
    """Demographics and other administrative information about a person or
    animal receiving care or other health-related services."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, other=None, type_=None):
        self.original_tagname_ = None
        super(Patient_Link, self).__init__(id, extension, modifierExtension, )
        self.other = other
        self.type_ = type_
    def factory(*args_, **kwargs_):
        if Patient_Link.subclass:
            return Patient_Link.subclass(*args_, **kwargs_)
        else:
            return Patient_Link(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_other(self): return self.other
    def set_other(self, other): self.other = other
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def hasContent_(self):
        if (
            self.other is not None or
            self.type_ is not None or
            super(Patient_Link, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Patient.Link', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Patient.Link')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Patient.Link', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Patient.Link'):
        super(Patient_Link, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Patient.Link')
    def exportChildren(self, outfile, level, namespace_='', name_='Patient.Link', fromsubclass_=False, pretty_print=True):
        super(Patient_Link, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.other is not None:
            self.other.export(outfile, level, namespace_, name_='other', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Patient.Link', mapping_=None):
        element = super(Patient_Link, self).to_etree(parent_element, name_, mapping_)
        if self.other is not None:
            other_ = self.other
            other_.to_etree(element, name_='other', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Patient_Link, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'other':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.other = obj_
            obj_.original_tagname_ = 'other'
        elif nodeName_ == 'type':
            obj_ = LinkType.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        super(Patient_Link, self).buildChildren(child_, node, nodeName_, True)
# end class Patient_Link


class LinkType(Element):
    """The type of link between this patient resource and another patient
    resource.If the element is present, it must have either a
    @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(LinkType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if LinkType.subclass:
            return LinkType.subclass(*args_, **kwargs_)
        else:
            return LinkType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(LinkType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='LinkType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='LinkType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='LinkType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='LinkType'):
        super(LinkType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='LinkType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='LinkType', fromsubclass_=False, pretty_print=True):
        super(LinkType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='LinkType', mapping_=None):
        element = super(LinkType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(LinkType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(LinkType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class LinkType



def find_record(model, query):
    '''Find a record and return it, or None'''
    try:
        return model.search(query, limit=1)[0]
    except:
        return None

def dumb_url_generate(args):
    """Generate url route with arguments

    params:
        args -- list of args to join
    returns:
        string (e.g., /Patient/test-3)
    """
    return ''.join(['/{0}/'.format(args[0]), '-'.join(map(str, args[1:]))])



from collections import namedtuple
from server.common import tryton
from server.health_fhir import health_Condition
from server.health_fhir import health_DiagnosticReport
from server.health_fhir import health_FamilyHistory
from server.health_fhir import health_Immunization
from server.health_fhir import health_MedicationStatement
from server.health_fhir import health_Observation
from server.health_fhir import health_Organization
from server.health_fhir import health_Practitioner
from server.health_fhir import health_Procedure
from server.health_fhir import health_Medication
from server.health_fhir import health_Patient
from server.health_fhir import Bundle
from server.health_fhir import health_Search

# Trying to keep consistent naming across all parts of the app
#   Consequently, bring all the pieces together and pass that around
Resource = namedtuple('Resource', ['model', 'adapter'])

mappings = {}

mappings['patient'] = Resource(
                    tryton.pool.get('gnuhealth.patient'),
                    health_Patient)

mappings['condition'] = Resource(
                    tryton.pool.get('gnuhealth.patient.disease'),
                    health_Condition)

mappings['diagnostic_report'] = Resource(
                    tryton.pool.get('gnuhealth.lab'),
                    health_DiagnosticReport)

mappings['family_history'] = Resource(
                    tryton.pool.get('gnuhealth.patient'), #Search from patient!
                    health_FamilyHistory)

mappings['immunization'] = Resource(
                    tryton.pool.get('gnuhealth.vaccination'),
                    health_Immunization)

mappings['medication'] = Resource(
                    tryton.pool.get('gnuhealth.medicament'),
                    health_Medication)

mappings['medication_statement'] = Resource(
                    tryton.pool.get('gnuhealth.patient.medication'),
                    health_MedicationStatement)

mappings['observation'] = Resource(
                    tryton.pool.get('gnuhealth.lab.test.critearea'),
                    health_Observation)

mappings['organization'] = Resource(
                    tryton.pool.get('gnuhealth.institution'),
                    health_Organization)

mappings['practitioner'] = Resource(
                    tryton.pool.get('gnuhealth.healthprofessional'),
                    health_Practitioner)

mappings['procedure'] = Resource(
                    tryton.pool.get('gnuhealth.operation'),
                    health_Procedure)

mappings['bundle'] = Resource(
                    None,
                    Bundle)

mappings['search'] = Resource(
                    None,
                    health_Search)

__all__ = ['mappings']



from .health_fhir_patient import Patient_Map
from .health_fhir_observation import Observation_Map
from .health_fhir_practitioner import Practitioner_Map
from .health_fhir_procedure import Procedure_Map
from .health_fhir_condition import Condition_Map 
from .health_fhir_diagnostic_report import DiagnosticReport_Map
from .health_fhir_family_history import FamilyHistory_Map
from .health_fhir_medication import Medication_Map
from .health_fhir_medication_statement import MedicationStatement_Map 
from .health_fhir_immunization import Immunization_Map 
from .health_fhir_organization import Organization_Map 
from server.common import safe_attrgetter
import re

# TODO: Add support for :missing modifier

class health_Search:
    """This class computes search queries to be used
    by Tryton models. In the future, hopefully add support
    for more complicated raw SQL queries.
    """

    def __init__(self, endpoint=None, request_args=None):
        """Create class

        Keyword arguments:
        endpoint -- name of endpoint (required!)
        """
        if endpoint is None:
            raise ValueError('Need endpoint value')
        if endpoint not in ('patient',
                            'observation',
                            'practitioner',
                            'procedure',
                            'diagnostic_report',
                            'condition',
                            'family_history',
                            'medication',
                            'medication_statement',
                            'immunization',
                            'organization'):
            raise ValueError('Not a valid endpoint')
        if request_args is None:
            raise ValueError('Need the request arguments!')

        self.request_args = request_args
        self.observation=Observation_Map()
        self.practitioner=Practitioner_Map()
        self.procedure=Procedure_Map()
        self.diagnostic_report=DiagnosticReport_Map()
        self.patient=Patient_Map()
        self.condition=Condition_Map()
        self.family_history=FamilyHistory_Map()
        self.medication=Medication_Map()
        self.medication_statement=MedicationStatement_Map()
        self.immunization=Immunization_Map()
        self.organization=Organization_Map()
        self.endpoint = getattr(self, endpoint)

        self.__get_dt_parser()
        self.valid_modifiers = { 'string': ['missing', 'exact'],
                                'token': ['missing', 'text'],
                                'number': ['missing'],
                                'date': ['missing'],
                                'reference': ['missing'], #:[type] from resources
                                'quantity': ['missing'] }

        self.valid_resources = [ 'Alert', 'AllergyIntolerance', 'CarePlan',
                'Composition', 'Condition', 'Conformance', 'Device',
                'DeviceObservationReport', 'DiagnosticOrder',
                'DiagnosticReport', 'DocumentReference', 'DocumentManifest',
                'Encounter', 'FamilyHistory', 'Group', 'ImagingStudy',
                'Immunization', 'ImmunizationRecommendation', 'List',
                'Location', 'Media', 'Medication', 'MedicationAdministration',
                'MedicationDispense', 'MedicationPrescription',
                'MedicationStatement', 'MessageHeader', 'Observation',
                'OperationOutcome', 'Order', 'OrderResponse', 'Organization',
                'Other', 'Patient', 'Practitioner', 'Procedure', 'Profile',
                'Provenance', 'Query', 'Questionnaire', 'RelatedPerson',
                'SecurityEvent', 'Specimen', 'Substance', 'Supply', 'ValueSet']

        self.type_parsers={'number': self.number_parser,
                    'date': self.date_parser,
                    'string': self.string_parser, 
                    'token': self.string_parser,
                    'quantity': self.quantity_parser,
                    'reference': self.string_parser}
                    #'user-defined': self.string_parser,
                    #'composite': self.string_parser}

    def __get_dt_parser(self):
        try:
            from dateutil.parser import parse
            def wrap_parse(string):
                '''Parser for date type.
                    Return ValueError, not TypeError
                '''
                try:
                    prefixes=('<=', '>=', '<', '>')
                    prefix, tmp = self.pop_prefix(string, prefixes)
                    split = self.split_string(tmp)
                    date=[parse(x) for x in split]
                    return (prefix, date)
                except:
                    raise ValueError
            self.date_parser=wrap_parse
        except:
            def fall_back(string):
                from time import strptime
                prefixes=('<=', '>=', '<', '>')
                prefix, tmp = self.pop_prefix(string, prefixes)
                split = self.split_string(tmp)
                date=[strptime(x, "%Y-%m-%dT%H:%M:%S") for x in split]
                return (prefix, date)
            self.date_parser=fallback

    def __is_valid_modifier(self, type_, modifier):
        if modifier is not None:
            modifiers = self.valid_modifiers.get(type_, None)
            if modifiers is not None:
                if type_ == 'resource':
                    modifiers.extend(self.valid_resources)
                if modifier in modifiers:
                    return True
            return False
        else:
            return True

    def split_string(self, string):
        """Split the string according to discrete
        search criteria (complicated!)

        Still work-in-progress!

        Keyword arguments:
        string -- the string to split
        """
        # FIX Handling \ is difficult:
        #    the string is already escaped against singleton \,
        #    but the standard is... uggh - do it simply now...
        #    don't allow escaped and non-escaped special characters
        #    in same search
        seps=(',', '$', '|')
        if ('\,' or '\$' or '\|') in string:
            unescaped = string.replace("\$", '$')
            unescaped = unescaped.replace("\,", ',')
            unescaped = unescaped.replace("\|", '|')
            return [unescaped]
        else:
            #Checked for allowed \, so if its singleton, must be invalid
            if (len(string) - len(string.replace('\\', ''))) % 2:
                raise ValueError
            else:
                return string.split(',')

    def pop_prefix(self, string, prefixes):
        """Pop the string prefix,
        returning (prefix, base)

        Keyword arguments:
        string -- string to parse
        prefixes -- prefixes to check
        """
        # FIX Unescaping equals... with good url handling
        #    becomes complicated since it will handle
        #    non-escaped equals fine
        for pre in prefixes:
            if string.startswith(pre):
                return (pre, string[len(pre):])
        return (None, string)

    def number_parser(self, string):
        """Parser for number type,
        returning (prefix, floats)

        Keyword arguments:
        string -- string to parse
        """
        prefixes=('<=', '>=', '<', '>')
        prefix, tmp = self.pop_prefix(string, prefixes)
        split = self.split_string(tmp)
        floats=[float(x) for x in split]
        return (prefix, floats)

    def quantity_parser(self, string):
        '''Parser for quantity type
            TODO: Handle ~
            TODO: Handle units, etc.'''
        prefixes=('<=', '>=', '<', '>')
        prefix, tmp = self.pop_prefix(string, prefixes)
        split = self.split_string(tmp)
        floats=[float(x) for x in split]
        return (prefix, floats)

    def string_parser(self, string):
        """Parser for string type

        Keyword arguments:
        string -- the string to parse
        """
        tmp = self.split_string(string)
        return (None, tmp)

    def __key_parameter_parser(self, string):
        """Parse key, including chains 
        (e.g., name:text, subject.name, subject:Patient.name)

        return: {'key': <name>,
                'modifier': <modifier>,
                'chains': [ <param>, ... ]}
        """

        m = string.split(':')
        if len(m) > 2:
            # Too many modifiers
            return None

        elif len(m) == 2:
            # There is a modifier, and possible chains
            tmp = m[1].split('.')
            d = { 'key': m[0],
                    'modifier': tmp[0],
                    'chains': tmp[1:] or None}

        else:
            # No modifier, possible chains
            chains = self.__pop_chain(m[0])
            d = { 'key': chains[0],
                    'modifier': None,
                    'chains': chains[1:] or None}
        return d

    def __pop_chain(self, string):
        """Naively pop chained arguments"""

        return string.split('.')

    def __parse_url_parameter(self, parameter):
        """Return parameter info"""
        key_info = self.__key_parameter_parser(parameter)
        # Is key in search_params?
        if key_info['key'] in self.endpoint.resource_search_params:
            #Great, now, is it supported and valid modifier?
            type_ = self.endpoint.resource_search_params[key_info['key']]
            if type_ is not None:
                if self.__is_valid_modifier(type_, key_info['modifier']):
                    # For resource types, semantic checking
                    #   later, but allow for now
                    if key_info['chains'] and type_ != 'reference':
                        # No chains with non-reference types
                        return None
                    key_info['type']=type_
                    return key_info
        # Bad modifier OR Non-supported OR bad parameter
        #   whatever, return None
        return None

    def chain_parameter_parser(self, info):
        """Take chain info, and return correct attr(s),
        returning (new_model_attrs, new_search_target_type)

        Keyword arguments:
        info -- search argument info
        """

        assert info['type'] == 'reference'
        attrs = []
        t=self.endpoint.search_mapping[info['key']]
        # Fix for searching from different start from expected model
        #  (e.g., FamilyHistory --> patient.search()
        if t:
            attrs.extend(t)
        target_resource = info['modifier'] or self.endpoint.chain_map[info['key']]
        while True:
            if not info['chains']:
                # Default to _id
                #TODO: In future, need to change for multi-model
                return (self.__make_attrs(attrs+['id']), 'token')
            if target_resource == 'Patient':
                current_map=self.patient
            elif target_resource == 'DiagnosticReport':
                current_map=self.diagnostic_report
            elif target_resource == 'Observation':
                current_map=self.observation
            elif target_resource == 'Practitioner':
                current_map=self.practitioner
            elif target_resource == 'Procedure':
                current_map=self.procedure
            elif target_resource == 'Condition':
                current_map=self.condition
            elif target_resource == 'FamilyHistory':
                current_map=self.family_history
            elif target_resource == 'Medication':
                current_map=self.medication
            elif target_resource == 'MedicationStatement':
                current_map=self.medication_statement
            elif target_resource == 'Immunization':
                current_map=self.immunization
            elif target_resource == 'Organization':
                current_map=self.organization
            else:
                raise ValueError('Unknown chain target')
            try:
                chain=info['chains'].pop(0)
                attrs.append(current_map.search_mapping[chain])
                target_resource=current_map.chain_map[chain]
            except KeyError:
                raise ValueError('Invalid parameter: non-reference chain or unsupported')
            except:
                break
        return (self.__make_attrs(attrs), current_map.resource_search_params[chain])

    def __make_attrs(self, l, current_attrs=[]):
        """Create attributes from chain lists"""
        if len(l) == 0: return current_attrs
        at = l.pop(0)
        if isinstance(at, basestring):
            return self.__make_attrs(l, ['.'.join((s,at)) for s in current_attrs] or [at])
        else:
            return self.__make_attrs(l, ['.'.join((l,a)) for l in current_attrs for a in at] or [a for a in at])

    def parse_url_string(self, request_args):
        """Parse url string

        Keyword arguments:
        request_args -- url arguments
        """

        full_search_info=[]
        for key, values in request_args.iterlists():
            if key in ['_count', 'page']: continue
            search_info = self.__parse_url_parameter(key)
            if search_info is not None:
                parser = self.type_parsers[search_info['type']]
                search_info['args']=[]
                for argument in values:
                    try:
                        arg_info = parser(argument)
                        search_info['args'].append({'prefix': arg_info[0],
                                                'value': arg_info[1]})
                    except:
                        raise ValueError('Bad argument: {}'.format(argument))
            else:
                raise ValueError('Bad parameter: {}'.format(key))
            full_search_info.append(search_info)

        return self.__validate_search_info(full_search_info)

    def __validate_search_info(self, search_info):
        """Validate the naive search parser against misleading constructions

        TODO: Check if there are the same keys, with different modifiers
           that are exclusionary
              e.g., [{ 'key': 'name', 'modifier': 'exact', ...},
         - Do not allow multiple AND values with exact modifier
         - Do not allow prefixes with multiple OR values
         - Do not allow parameter without argument

         Keyword arguments:
         search_info -- parsed search arguments
        """

        for d in search_info:
            for arg in d['args']:
                if arg['prefix'] and len(arg['value']) > 1:
                    raise ValueError('Do not allow prefixes with multiple OR values')
                for a in arg['value']:
                    if not a:
                        raise ValueError('Do not allow parameters without values')
            if d['modifier'] == 'exact':
                if len(d['args']) > 1:
                    raise ValueError('Do not allow :exact with multiple AND values')
        return search_info

    def single_model_query_generate(self, full_search_info):
        """For single model searches (no weird outer joins, etc.)

        This function is bloated

        Keywoard arguments:
        full_search_info -- fully parsed search argumnts

        Returns:
        search domain -- (e.g., [('name', 'ilike', '%e%')])
        """

        full_query=[]
        #Look for key or key+modifier in the search_mapping
        for query in full_search_info:
            #Look for key+modifier first
            try:
                keymod = ':'.join([query['key'], query['modifier']])
            except:
                keymod = None

            if keymod and keymod in self.endpoint.search_mapping:

                model_attr=self.endpoint.search_mapping[keymod]

                #Only text modifiers on a different model attr
                #  get here (e.g., <token>:text)
                operator = 'ilike'

            else:
                if query['type'] == 'reference':
                    # Override query information with new info
                    model_attr, query['type']=self.chain_parameter_parser(query)
                    query['chains']=None
                else:
                    model_attr=self.endpoint.search_mapping[query['key']]
                if query['type'] == 'string':
                    if query['modifier'] == 'exact':
                        operator = '='
                    else:
                        operator = 'ilike'
                elif query['type'] == 'token':
                    if query['modifier'] == 'text':
                        operator = 'ilike'
                    else:
                        operator = '='
                else:
                    # Figure out later (could be 'in')
                    operator = '='

            #Now, handle multiple model_attrs and multiple values
            # These are AND
            and_queries=[]
            for arg in query['args']:
                or_queries=[]
                MULTIPLE=False
                if len(model_attr) > 1:
                    MULTIPLE=True
                #These are OR
                for attr in model_attr:
                    if len(arg['value']) > 1:
                        if operator == '=':
                            q = [(attr, 'in' ,arg['value'])]
                        else:
                            #ilike needs to do regex thing
                            if arg['prefix']:
                                q=[[(attr, arg['prefix'], a)] for a in arg['value']]
                            else:
                                if operator == 'ilike':
                                    q=[[(attr, operator, '{0}{1}{2}'.format('%',str(a),'%'))] for a in arg['value']]
                                else:
                                    q=[[(attr, operator, a)] for a in arg['value']]
                    else:
                        #ilike needs to do regex thing
                        if arg['prefix']:
                            q = [(attr, arg['prefix'],arg['value'][0])]
                        else:
                            if operator == 'ilike':
                                q = [(attr, operator ,'{0}{1}{2}'.format('%',str(arg['value'][0]),'%'))]
                            else:
                                q = [(attr, operator,arg['value'][0])]

                    # If there are multiple model attributes
                    #    AND only one statemement, then it
                    #    will be OR'd later...
                    if MULTIPLE == True and len(q) == 1:
                        or_queries.append(q)
                    else:
                        or_queries.extend(q)

                #If only one or_query, then cleanup
                #  Not absolutely necessary, but cleaner query
                if len(or_queries) == 1:
                    and_queries.extend(or_queries)
                else:
                    t=['OR']
                    t.extend(or_queries)
                    and_queries.append(t)
            full_query.extend(and_queries)

        # Extend with root search
        full_query.extend(getattr(self.endpoint, 'root_search', []))

        return full_query

    @property
    def query(self):
        """Get the search query

        For now, only single model support
        """

        full_search_info=self.parse_url_string(self.request_args)
        return self.single_model_query_generate(full_search_info)



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from trytond.model import ModelView, ModelSingleton, ModelSQL, fields
from trytond.pyson import Eval, Not, Bool, PYSONEncoder
from trytond.pool import Pool


__all__ = ['GnuHealthSequences', 'DengueDUSurvey']


class GnuHealthSequences(ModelSingleton, ModelSQL, ModelView):
    __name__ = 'gnuhealth.sequences'

    dengue_du_survey_sequence = fields.Property(fields.Many2One(
        'ir.sequence',
        'Dengue Survey Sequence', required=True,
        domain=[('code', '=', 'gnuhealth.dengue_du_survey')]))


class DengueDUSurvey(ModelSQL, ModelView):
    'Dengue DU Survey'
    __name__ = 'gnuhealth.dengue_du_survey'

    name = fields.Char('Survey Code', readonly=True)
    du = fields.Many2One('gnuhealth.du', 'DU', help="Domiciliary Unit")
    survey_date = fields.Date('Date', required=True)

    du_status = fields.Selection([
        (None, ''),
        ('initial', 'Initial'),
        ('unchanged', 'Unchanged'),
        ('better', 'Improved'),
        ('worse', 'Worsen'),
        ], 'Status',
        help="DU status compared to last visit", required=True, sort=False)

    # Surveillance traps (ovitraps)
    ovitraps = fields.Boolean(
        'Ovitraps',
        help="Check if ovitraps are in place")

    # Current issues
    aedes_larva = fields.Boolean(
        'Larvae', "Check this box if Aedes aegypti larvae were found")
    larva_in_house = fields.Boolean(
        'Domiciliary',
        help="Check this box if larvae were found inside the house")
    larva_peri = fields.Boolean(
        'Peri-Domiciliary',
        help="Check this box if larva were found in the peridomiciliary area")

    old_tyres = fields.Boolean('Tyres', help="Old vehicle tyres found")

    animal_water_container = fields.Boolean(
        'Animal Water containers',
        help="Animal water containers not scrubbed or clean")

    flower_vase = fields.Boolean(
        'Flower vase',
        help="Flower vases without scrubbing or cleaning")

    potted_plant = fields.Boolean(
        'Potted Plants',
        help="Potted Plants with saucers")

    tree_holes = fields.Boolean(
        'Tree holes',
        help="unfilled tree holes")

    rock_holes = fields.Boolean(
        'Rock holes',
        help="unfilled rock holes")

    # Chemical controls for adult mosquitoes

    du_fumigation = fields.Boolean(
        'Fumigation', help="The DU has been fumigated")
    fumigation_date = fields.Date(
        'Fumigation Date', help="Last Fumigation Date",
        states={'invisible': Not(Bool(Eval('du_fumigation')))})

    observations = fields.Text('Observations')
    next_survey_date = fields.Date('Next survey')

    @staticmethod
    def default_survey_date():
        return datetime.now()

    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('name'):
                config = Config(1)
                values['name'] = Sequence.get_id(
                    config.dengue_du_survey_sequence.id)

        return super(DengueDUSurvey, cls).create(vlist)



from flask import Flask, g
from flask_login import current_user
from server.common import tryton, login_manager, Api, recordConverter

def before_request():
    """Allows access to user object
        for every request
    """
    g.user = current_user

def create_app(config=None):
    app = Flask(__name__)
    app.config.from_object('server.config.DebugConfig')
    if config is not None:
        app.config.from_object(config)
    app.config.from_envvar('FHIR_SERVER_CONFIG', silent=True)

    with app.app_context():
        import logging
        logging.basicConfig(filename='flask.log', level=logging.INFO,
                format='%(asctime)s:::%(levelname)s:::%(name)s: %(message)s')

        # Initialize tryton
        tryton.init_app(app)

        # Login
        login_manager.init_app(app)
        login_manager.login_view='auth_endpoint.login'
        login_manager.session_protection='strong'

        # Initiate the REST API
        api=Api(app)

        # The user model
        user = tryton.pool.get('res.user')

        # Setup tryton config
        @tryton.default_context
        def default_context():
            return user.get_preferences(context_only=True)

        # Add Model-ID-Field url converter
        app.url_map.converters['item']=recordConverter

        # Store current user in g.user
        app.before_request(before_request)

        #### ADD THE ROUTES ####
        from server.resources.system import Conformance
        api.add_resource(Conformance, '/', '/metadata')

        def add_fhir_routes(resource):
            """Adds complex FHIR routes"""

            for _, route in resource.routing_table.items():
                api.add_resource(route['handler'], *route['uri'])

        from server.resources.patient import routing as pat
        add_fhir_routes(pat)

        from server.resources.diagnostic_report import routing as dr
        add_fhir_routes(dr)

        from server.resources.observation import routing as obs
        add_fhir_routes(obs)

        from server.resources.practitioner import routing as hp
        add_fhir_routes(hp)

        from server.resources.procedure import routing as op
        add_fhir_routes(op)

        from server.resources.condition import routing as condition
        add_fhir_routes(condition)

        from server.resources.family_history import routing as fh
        add_fhir_routes(fh)

        from server.resources.medication import routing as med
        add_fhir_routes(med)

        from server.resources.medication_statement import routing as ms
        add_fhir_routes(ms)

        from server.resources.immunization import routing as imm
        add_fhir_routes(imm)

        from server.resources.organization import routing as org
        add_fhir_routes(org)

        # Handle the authentication blueprint
        #   TODO: Use OAuth or something robust
        from server.resources.auth import auth_endpoint
        app.register_blueprint(auth_endpoint, url_prefix='/auth')

    return app



import re
import os
import sys
import time
import shutil
import tarfile
import tempfile
import unittest
import subprocess

testdir = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.join(testdir, '..'))

import pywebdav.server.server

# Run davserver
user = 'test'
password = 'pass'
port = 38028


class Test(unittest.TestCase):
    def setUp(self):

        self.rundir = tempfile.mkdtemp()
        self._ensure_litmus()

    def _ensure_litmus(self):

        litmus_dist = os.path.join(testdir, 'litmus-0.13')
        self.litmus = os.path.join(litmus_dist, 'litmus')
        if not os.path.exists(self.litmus):
            print('Compiling litmus test suite')

            if os.path.exists(litmus_dist):
                shutil.rmtree(litmus_dist)
            with tarfile.open(litmus_dist + '.tar.gz') as tf:
                tf.extractall(path=testdir)
            ret = subprocess.call(['sh', './configure'], cwd=litmus_dist)
            # assert ret == 0
            ret = subprocess.call(['make'], cwd=litmus_dist)
            # assert ret == 0
            litmus = os.path.join(litmus_dist, 'litmus')
            # assert os.path.exists(litmus)

    def tearDown(self):
        print("Cleaning up tempdir")
        shutil.rmtree(self.rundir)

    def test_run_litmus(self):

        result = []
        proc = None
        try:
            print('Starting davserver')
            davserver_cmd = [sys.executable, os.path.join(testdir, '..', 'pywebdav', 'server', 'server.py'), '-D',
                             self.rundir, '-u', user, '-p', password, '-H', 'localhost', '--port', str(port)]
            self.davserver_proc = subprocess.Popen(davserver_cmd)
            # Ensure davserver has time to startup
            time.sleep(1)

            # Run Litmus
            print('Running litmus')
            try:
                ret = subprocess.call([self.litmus, 'http://localhost:%d' % port, user, password])
                results = subprocess.check_output([self.litmus, 'http://localhost:%d' % port, user, password])
            except subprocess.CalledProcessError as ex:
                results = ex.output
            lines = results.decode().split('\n')
            assert len(lines), "No litmus output"
            for line in lines:
                line = line.split('\r')[-1]
                result.append(line)
                if len(re.findall('^ *\d+\.', line)):
                    assert line.endswith('pass'), line

        finally:
            print('\n'.join(result))

            print('Stopping davserver')
            self.davserver_proc.kill()


    def test_run_litmus_noauth(self):

        result = []
        proc = None
        try:
            print('Starting davserver')
            davserver_cmd = [sys.executable, os.path.join(testdir, '..', 'pywebdav', 'server', 'server.py'), '-D',
                             self.rundir, '-n', '-H', 'localhost', '--port', str(port)]
            self.davserver_proc = subprocess.Popen(davserver_cmd)
            # Ensure davserver has time to startup
            time.sleep(1)

            # Run Litmus
            print('Running litmus')
            try:
                ret = subprocess.call([self.litmus, 'http://localhost:%d' % port])
                results = subprocess.check_output([self.litmus, 'http://localhost:%d' % port])
            except subprocess.CalledProcessError as ex:
                results = ex.output
            lines = results.decode().split('\n')
            assert len(lines), "No litmus output"
            for line in lines:
                line = line.split('\r')[-1]
                result.append(line)
                if len(re.findall('^ *\d+\.', line)):
                    assert line.endswith('pass'), line

        finally:
            print('\n'.join(result))

            print('Stopping davserver')
            self.davserver_proc.kill()



from .base_classes import *
from .support_functions import *


class Media(Resource):
    """A photo, video, or audio recording acquired or used in healthcare.
    The actual content may be inline or provided by direct
    reference.If the element is present, it must have either a
    @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, type_=None, subtype=None, identifier=None, dateTime=None, subject=None, operator=None, view=None, deviceName=None, height=None, width=None, frames=None, length=None, content=None):
        self.original_tagname_ = None
        super(Media, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.type_ = type_
        self.subtype = subtype
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.dateTime = dateTime
        self.subject = subject
        self.operator = operator
        self.view = view
        self.deviceName = deviceName
        self.height = height
        self.width = width
        self.frames = frames
        self.length = length
        self.content = content
    def factory(*args_, **kwargs_):
        if Media.subclass:
            return Media.subclass(*args_, **kwargs_)
        else:
            return Media(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_subtype(self): return self.subtype
    def set_subtype(self, subtype): self.subtype = subtype
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_dateTime(self): return self.dateTime
    def set_dateTime(self, dateTime): self.dateTime = dateTime
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_operator(self): return self.operator
    def set_operator(self, operator): self.operator = operator
    def get_view(self): return self.view
    def set_view(self, view): self.view = view
    def get_deviceName(self): return self.deviceName
    def set_deviceName(self, deviceName): self.deviceName = deviceName
    def get_height(self): return self.height
    def set_height(self, height): self.height = height
    def get_width(self): return self.width
    def set_width(self, width): self.width = width
    def get_frames(self): return self.frames
    def set_frames(self, frames): self.frames = frames
    def get_length(self): return self.length
    def set_length(self, length): self.length = length
    def get_content(self): return self.content
    def set_content(self, content): self.content = content
    def hasContent_(self):
        if (
            self.type_ is not None or
            self.subtype is not None or
            self.identifier or
            self.dateTime is not None or
            self.subject is not None or
            self.operator is not None or
            self.view is not None or
            self.deviceName is not None or
            self.height is not None or
            self.width is not None or
            self.frames is not None or
            self.length is not None or
            self.content is not None or
            super(Media, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Media', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Media')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Media', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Media'):
        super(Media, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Media')
    def exportChildren(self, outfile, level, namespace_='', name_='Media', fromsubclass_=False, pretty_print=True):
        super(Media, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.subtype is not None:
            self.subtype.export(outfile, level, namespace_, name_='subtype', pretty_print=pretty_print)
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.dateTime is not None:
            self.dateTime.export(outfile, level, namespace_, name_='dateTime', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.operator is not None:
            self.operator.export(outfile, level, namespace_, name_='operator', pretty_print=pretty_print)
        if self.view is not None:
            self.view.export(outfile, level, namespace_, name_='view', pretty_print=pretty_print)
        if self.deviceName is not None:
            self.deviceName.export(outfile, level, namespace_, name_='deviceName', pretty_print=pretty_print)
        if self.height is not None:
            self.height.export(outfile, level, namespace_, name_='height', pretty_print=pretty_print)
        if self.width is not None:
            self.width.export(outfile, level, namespace_, name_='width', pretty_print=pretty_print)
        if self.frames is not None:
            self.frames.export(outfile, level, namespace_, name_='frames', pretty_print=pretty_print)
        if self.length is not None:
            self.length.export(outfile, level, namespace_, name_='length', pretty_print=pretty_print)
        if self.content is not None:
            self.content.export(outfile, level, namespace_, name_='content', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Media', mapping_=None):
        element = super(Media, self).to_etree(parent_element, name_, mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.subtype is not None:
            subtype_ = self.subtype
            subtype_.to_etree(element, name_='subtype', mapping_=mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.dateTime is not None:
            dateTime_ = self.dateTime
            dateTime_.to_etree(element, name_='dateTime', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.operator is not None:
            operator_ = self.operator
            operator_.to_etree(element, name_='operator', mapping_=mapping_)
        if self.view is not None:
            view_ = self.view
            view_.to_etree(element, name_='view', mapping_=mapping_)
        if self.deviceName is not None:
            deviceName_ = self.deviceName
            deviceName_.to_etree(element, name_='deviceName', mapping_=mapping_)
        if self.height is not None:
            height_ = self.height
            height_.to_etree(element, name_='height', mapping_=mapping_)
        if self.width is not None:
            width_ = self.width
            width_.to_etree(element, name_='width', mapping_=mapping_)
        if self.frames is not None:
            frames_ = self.frames
            frames_.to_etree(element, name_='frames', mapping_=mapping_)
        if self.length is not None:
            length_ = self.length
            length_.to_etree(element, name_='length', mapping_=mapping_)
        if self.content is not None:
            content_ = self.content
            content_.to_etree(element, name_='content', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Media, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = MediaType.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'subtype':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.subtype = obj_
            obj_.original_tagname_ = 'subtype'
        elif nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'dateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.dateTime = obj_
            obj_.original_tagname_ = 'dateTime'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'operator':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.operator = obj_
            obj_.original_tagname_ = 'operator'
        elif nodeName_ == 'view':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.view = obj_
            obj_.original_tagname_ = 'view'
        elif nodeName_ == 'deviceName':
            obj_ = string.factory()
            obj_.build(child_)
            self.deviceName = obj_
            obj_.original_tagname_ = 'deviceName'
        elif nodeName_ == 'height':
            obj_ = integer.factory()
            obj_.build(child_)
            self.height = obj_
            obj_.original_tagname_ = 'height'
        elif nodeName_ == 'width':
            obj_ = integer.factory()
            obj_.build(child_)
            self.width = obj_
            obj_.original_tagname_ = 'width'
        elif nodeName_ == 'frames':
            obj_ = integer.factory()
            obj_.build(child_)
            self.frames = obj_
            obj_.original_tagname_ = 'frames'
        elif nodeName_ == 'length':
            obj_ = integer.factory()
            obj_.build(child_)
            self.length = obj_
            obj_.original_tagname_ = 'length'
        elif nodeName_ == 'content':
            obj_ = Attachment.factory()
            obj_.build(child_)
            self.content = obj_
            obj_.original_tagname_ = 'content'
        super(Media, self).buildChildren(child_, node, nodeName_, True)
# end class Media


class MediaType(Element):
    """Whether the Media is a photo, video, or audioIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(MediaType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if MediaType.subclass:
            return MediaType.subclass(*args_, **kwargs_)
        else:
            return MediaType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(MediaType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MediaType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MediaType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MediaType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MediaType'):
        super(MediaType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MediaType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='MediaType', fromsubclass_=False, pretty_print=True):
        super(MediaType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MediaType', mapping_=None):
        element = super(MediaType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(MediaType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(MediaType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class MediaType
